@@ -4,14 +4,16 @@ use 5.005;
 
 =head1 NAME
 
-MIME::EncWords - deal with RFC-1522 encoded words (improved)
+MIME::EncWords - deal with RFC 2047 encoded words (improved)
 
 =head1 SYNOPSIS
 
 I<L<MIME::EncWords> is aimed to be another implimentation
 of L<MIME::Words> so that it will achive more exact conformance with
-MIME specifications.  Additionally, it contains some improvements.
-Following synopsis and descriptions are inherited from its inspirer.>
+RFC 2047 (former RFC 1522) specifications.  Additionally, it contains
+some improvements.
+Following synopsis and descriptions are inherited from its inspirer,
+with description of improvements and clarifications added.>
 
 Before reading further, you should see L<MIME::Tools> to make sure that
 you understand where this module fits into the grand scheme of things.
@@ -117,7 +119,7 @@ if (MIME::Charset::USE_ENCODE) {
 #------------------------------
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = "0.03";
+$VERSION = '1.000';
 
 ### Nonprintables (controls + x7F + 8bit):
 #my $NONPRINT = "\\x00-\\x1F\\x7F-\\xFF";
@@ -224,6 +226,17 @@ B<Note>:
 This feature is still information-lossy, I<except> when C<"_UNICODE_"> is
 specified.
 
+=item Detect7bit
+
+B<Improvement by this modlue>:
+Try to detect 7-bit charset on unencoded portions.
+Default is C<"YES">.
+When Unicode/multibyte support is disabled,
+this option will not have any effects
+(see L<MIME::Charset/USE_ENCODE>).
+B<Note>:
+This feature was introduced at release 1.000.
+
 =item Field
 
 Name of the mail field this string came from.  I<Currently ignored.>
@@ -247,6 +260,7 @@ sub decode_mimewords {
     my $encstr = shift;
     my %params = @_;
     my $cset = $params{"Charset"};
+    my $detect7bit = uc($params{'Detect7bit'} || "YES");
     my @tokens;
     $@ = '';           ### error-return
 
@@ -320,6 +334,19 @@ sub decode_mimewords {
     }
     push @tokens, [$spc] if $spc;
 
+    # Detect 7-bit charset
+    if ($detect7bit ne "NO") {
+	foreach my $t (@tokens) {
+	    unless ($t->[1]) {
+		my $charset = &MIME::Charset::_detect_7bit_charset($t->[0]);
+		if ($charset and $charset ne &MIME::Charset::default()) {
+		    $t->[0] =~ s/[\r\n\t ]+/ /g;
+		    $t->[1] = $charset;
+		}
+	    }
+	}
+    }
+
     return (wantarray ? @tokens : join('',map {
 	&_convert($_->[0], $_->[1], $cset)
 	} @tokens));
@@ -403,13 +430,13 @@ sub encode_mimeword {
     if ($encoding eq 'Q') {
 	$encstr = &_encode_Q($word);
     } elsif ($encoding eq "S") {
-	if (encoded_header_len($word, "B", $charset) <
-	    encoded_header_len($word, "Q", $charset)) {
+	my ($B, $Q) = (&_encode_B($word), &_encode_Q($word));
+	if (length($B) < length($Q)) {
 	    $encoding = "B";
-	    $encstr = &_encode_B($word);
+	    $encstr = $B;
 	} else {
 	    $encoding = "Q";
-	    $encstr = &_encode_Q($word);
+	    $encstr = $Q;
 	}
     } else { # "B"
 	$encoding = "B";
@@ -437,12 +464,7 @@ RAW may be a Unicode string when Unicode/multibyte support is enabled
 (see L<MIME::Charset/USE_ENCODE>).
 Furthermore, RAW may be a reference to that returned
 by L<"decode_mimewords"> on array context.  In latter case "Charset"
-option (see below) will be overridden.
-
-B<Change by this module>:
-Adjacent encoded-words are concatenated.  Then they are splitted taking
-care of character boundaries of multibyte sequences, when Unicode/multibyte
-support is enabled.
+option (see below) will be overridden (see also notes below).
 
 Any arguments past the RAW string are taken to define a hash of options:
 
@@ -479,8 +501,39 @@ B<Improvement by this module>:
 Length of mail field name will be considered in the first line of
 encoded header.
 
+=item Minimal
+
+B<Improvement by this module>:
+Takes care of natural word separators (i.e. whitespaces)
+in the text to be encoded.
+If C<"NO"> is specified, this module will encode whole text
+(if encoding needed) not regarding whitespaces;
+encoded-words exceeding line length will be splitted based only on their
+lengths.
+Default is C<"YES">.
+
+B<Note>:
+As of release 0.040, default has been changed to C<"YES"> to ensure
+compatibility with MIME::Words.
+On earlier releases, this option was fixed to be C<"NO">.
+
+=item Replacement
+
+B<Improvement by this module>:
+See L<MIME::Charset/ERROR HANDLING>.
+B<Note>:
+This feature was introduced at release 1.000.
+
 =back
 
+B<Notes on improvement by this module>:
+When RAW is an arrayref,
+adjacent encoded-words are concatenated.  Then they are splitted taking
+care of character boundaries of multibyte sequences, when Unicode/multibyte
+support is enabled.
+Portions for unencoded data should include surrounding whitespace(s), or
+they will be merged into adjoining encoded word(s).
+
 =cut
 
 sub encode_mimewords  {
@@ -490,13 +543,20 @@ sub encode_mimewords  {
     my $detect7bit = uc($params{'Detect7bit'} || "YES");
     my $encoding = uc($params{'Encoding'});
     my $header_name = $params{'Field'};
+    my $minimal = uc($params{'Minimal'} || "YES");
+    my $replacement = uc($params{'Replacement'} || 'DEFAULT');
     my $firstlinelen = $MAXLINELEN;
     if ($header_name) {
 	$firstlinelen -= length($header_name.': ');
     }
 
     unless (ref($words) eq "ARRAY") {
-	$words = [[$words, $charset]];
+	if ($minimal eq "YES") {
+	    my @words = map {[$_, $charset]} split(/((?:\A|[\t ])[\t \x21-\x7E]+(?:[\t ]|\Z))/, $words);
+	    $words = \@words;
+	} else {
+	    $words = [[$words, $charset]];
+	}
     }
 
     # Translate / concatenate words.
@@ -513,18 +573,25 @@ sub encode_mimewords  {
 	    unless (resolve_alias($cset)) {
 		if ($s !~ $UNSAFE) {
 		    $cset = "US-ASCII";
+		} elsif ($replacement =~ '^(CROAK|STRICT)$') {
+		    croak "MIME::EncWords: unsupported charset: $cset\n";
 		} else {
 		    $cset = "UTF-8";
 		}
 	    }
-	    $s = encode($cset, $s);
+	    if ($replacement =~ /^(CROAK|STRICT)$/) {
+		$s = encode($cset, $s, FB_CROAK());
+	    } else {
+		$s = encode($cset, $s);
+	    }
 	}
 
 	# Determine charset and encoding.
 	if ($encoding eq "A") {
 	    ($s, $cset, $enc) =
 		header_encode($s, $cset || $charset,
-			      Detect7bit => $detect7bit);
+			      Detect7bit => $detect7bit,
+			      Replacement => $replacement);
 	} else {
 	    $cset ||= ($charset || ($s !~ $UNSAFE)? "US-ASCII": "ISO-8859-1");
 	    $enc = $encoding ||
@@ -773,6 +840,13 @@ sub _clip_unsafe {
 
 =back
 
+=head1 VERSION
+
+Consult $VERSION variable.
+
+Development versions of this module may be found at
+L<http://hatuka.nezumi.nu/repos/MIME-EncWords/>.
+
 =head1 SEE ALSO
 
 L<MIME::Charset>,
